# Пример 2. Как создать электронный магазин при помощи Limb
## Цели данного примера

* Показать использование пакета WEB_APP на реальной задаче.
* Показать, как в Limb3 интегрируются пакеты MACRO, DBAL, ACTIVE_RECORD, WEB_APP и CMS.
* Рассказать о внутреннем устройстве приложения, выполненном на базе Limb3.

Код данного примера можно найти в разделе c примерами использования Limb3

## Необходимые знания
Мы полагаем, что мы уже выполнили [первый пример создания приложения на базе Limb3](./basic.md) и имеете начальное представление, что такое ACTIVE_RECORD и MACRO.

Плюсом будет, если вы кратко познакомитесь с возможностями следующих пакетов:

* [ACTIVE_RECORD](../../../active_record/docs/ru/active_record.md)
* [MACRO](../../../macro/docs/ru/macro.md)
* [CMS](../../../cms/docs/ru/cms.md)

Особо вчитываться пока может и не стоит. В процессе описания примера мы постараемся указывать на все сложные и новые моменты, которые могут быть для вас незнакомыми по сравнению с первым примером. Если что-то требует большого описания, мы будем давать ссылки на соответствующие страницы документации.

## Что из себя будет представлять пример
Пример — это электронный магазин, который будет позволять пользователям делать покупки из небольшого списка товаров и отслеживать состояние заказа. Приложение предусматривает регистрацию пользователей, разделение пользователей на покупателей и администраторов, хранение состояний в сессии, поиск по товарам и многое другое. Все это позволит рассмотреть Limb с различных сторон, а также показать некоторые типичные приемы работы с ним.

Мы не будем разрабатывать административную панель с нуля, а возьмем для этого пакет [CMS](../../../cms/docs/ru/cms.md).

## Системные требования

* Операционная система: *nix, Windows 98/ME/2000/XP
* Веб-сервер: Apache 1.3+ (с подключенным модулем mod_rewrite)
* PHP: PHP 5.1.4+
* Сервер базы данных MySQL 4.1+ или БД [SQLite](http://sqlite.org/)

Если ваш веб-сервер обслуживает несколько доменов, то проект необходимо будет зарегистрировать в списке виртуальных хостов. Иначе же надо настроить DocumentRoot на директорию, где находятся файлы проекта.

## Необходимые пакеты Limb
Для выполнения данного примера нам необходим пакеты [CMS](../../../cms/docs/ru/cms.md), [WEB_APP](../../../web_app/docs/ru/web_app.md), [WYSIWYG](../../../wysiwyg/docs/ru/wysiwyg.md), а также пакеты, от которых они зависят. Информацию о том, как получить пакеты Limb в разделе [«Где и как скачать Limb3»](../how_to_download.md).

## Где взять готовое приложение?
Исходные коды примера можно получить с сайта GitHub — http://github.com/limb-php-framework/limb-example-shop/downloads

Мы будем использовать строку **shop** для указания пути, где лежат файлы готового приложения. Они могут нам потребоваться в процессе выполнения примера — мы будем брать оттуда некоторые файлы изображений, css-файлы и некоторые другие

## Далее

1. [Шаг 1. Более подробно о функциональности приложения](./shop/step1.md)
2. [Шаг 2. Запуск приложения. База данных. Базовые шаблоны](./shop/step2.md)
3. [Шаг 3. Пользователи, и все, что с ними связано](./shop/step3.md)
  * [Шаг 3.1 Регистрация пользователей](./shop/step3-1.md)
  * [Шаг 3.2 Аутентификация пользователей](./shop/step3-2.md)
  * [Шаг 3.3 Отображение профайла пользователя](./shop/step3-3.md)
  * [Шаг 3.4 Управление списком пользователей](./shop/step3-4.md)
4. [Шаг 4. Создание и отображение списка товаров для администраторов](./shop/step4.md)
5. [Шаг 5. Отображение списка товаров для покупателей. Поиск товаров](./shop/step5.md)
6. [Шаг 6. Работа покупателей с корзиной заказа](./shop/step6.md)
7. [Шаг 7. Оформление и сохранение заказа в базе данных](./shop/step7.md)
  * [Шаг 7.1. Модель, связанная с заказами](./shop/step7-1.md)
  * [Шаг 7.2. Оформление заказа](./shop/step7-2.md)
8. [Шаг 8. Работа с заказами](./shop/step8.md)
9. [Шаг 9. Рекомендации по дальнейшему изучению](./shop/step9.md)
